@@ -7,6 +7,7 @@ import cgi
 import sys
 import json
 import codecs
+import pprint
 import urllib
 
 
@@ -14,7 +15,7 @@ PY3 = (sys.version_info[0] == 3)
 if PY3:
     unicode, basestring = str, str
 
-__version__ = '0.7.0'
+__version__ = '0.7.1'
 __author__ = 'Mahmoud Hashemi'
 __contact__ = 'mahmoudrhashemi@gmail.com'
 __url__ = 'https://github.com/mahmoud/ashes'
@@ -28,7 +29,7 @@ node_re = re.compile(r'({'
                      r'(?:(?P<symbol>[\~\#\?\@\:\<\>\+\^\%])\s*)?'
                      r'(?P<refpath>[a-zA-Z0-9_\$\.]+|"[^"]+")'
                      r'(?:\:(?P<contpath>[a-zA-Z0-9\$\.]+))?'
-                     r'(?P<filters>\|[a-z]+)*?'
+                     r'(?P<filters>[\|a-z]+)*?'
                      r'(?P<params>(?:\s+\w+\=(("[^"]*?")|([\w\.]+)))*)?'
                      r'\s*'
                      r'(?P<selfclosing>\/)?'
@@ -304,8 +305,6 @@ def get_tag(match, inline=False):
     symbol = groups['symbol']
     closing = groups['closing']
     refpath = groups['refpath']
-    if symbol and len(symbol) > 1:
-        import pdb;pdb.set_trace()
     if closing:
         tag_type = ClosingTag
     elif symbol is None and refpath is not None:
@@ -897,6 +896,21 @@ def comma_num(val):
         return str(val)
 
 
+def pp_filter(val):
+    return pprint.pformat(val)
+
+
+JSON_PP_INDENT = 2
+
+
+def ppjson_filter(val):
+    "A best-effort pretty-printing filter, based on the JSON module"
+    try:
+        return json.dumps(val, indent=JSON_PP_INDENT, sort_keys=True)
+    except TypeError:
+        return unicode(val)
+
+
 # Helpers
 
 def first_helper(chunk, context, bodies, params=None):
@@ -950,10 +964,12 @@ def _do_compare(chunk, context, bodies, params, cmp_op):
         body = bodies['block']
         key = params['key']
         value = params['value']
-        typestr = params.get('type', 'string')
+        typestr = params.get('type')
     except KeyError:
         return chunk
     rkey = _resolve_value(key, chunk, context)
+    if not typestr:
+        typestr = _COERCE_REV_MAP.get(type(rkey), 'string')
     rvalue = _resolve_value(value, chunk, context)
     crkey, crvalue = _coerce(rkey, typestr), _coerce(rvalue, typestr)
     if isinstance(crvalue, type(crkey)) and cmp_op(crkey, crvalue):
@@ -979,6 +995,12 @@ _COERCE_MAP = {
     'string': unicode,
     'boolean': bool,
 }  # Not implemented: date, context
+_COERCE_REV_MAP = dict([(v, k) for k, v in _COERCE_MAP.items()])
+_COERCE_REV_MAP[int] = 'number'
+try:
+    _COERCE_REV_MAP[long] = 'number'
+except NameError:
+    pass
 
 
 def _coerce(value, typestr):
@@ -1528,7 +1550,9 @@ DEFAULT_FILTERS = {
     'j': escape_js,
     'u': escape_uri,
     'uc': escape_uri_component,
-    'cn': comma_num}
+    'cn': comma_num,
+    'pp': pp_filter,
+    'ppjson': ppjson_filter}
 
 
 #########
@@ -1639,6 +1663,13 @@ class Template(object):
             return Compiler(self.env)._gen_python(ast)
         return func
 
+    def __repr__(self):
+        cn = self.__class__.__name__
+        name, source_file = self.name, self.source_file
+        if not source_file:
+            return '<%s name=%r>' % (cn, name)
+        return '<%s name=%r source_file=%r>' % (cn, name, source_file)
+
 
 class AshesException(Exception):
     pass
@@ -1963,6 +1994,10 @@ def _main():
     ae.register_source('tmpl', '{`{ok}thing`}')
     print(ae.render('tmpl', {'thing': 21000}))
 
+    ae.register_source('tmpl2', '{test|pp}')
+    out = ae.render('tmpl2', {'test': ['<hi>'] * 10})
+    print(out)
+
 
 if __name__ == '__main__':
     _main()
